package ar.rulosoft.mimanganu.servers;

import android.content.Context;

import com.squareup.duktape.Duktape;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ar.rulosoft.mimanganu.R;
import ar.rulosoft.mimanganu.componentes.Chapter;
import ar.rulosoft.mimanganu.componentes.Manga;
import ar.rulosoft.mimanganu.componentes.ServerFilter;
import ar.rulosoft.mimanganu.utils.Util;
import ar.rulosoft.navegadores.Navigator;

class KissManga extends ServerBase {
    private static final String HOST = "http://kissmanga.com";

    private static final String PATTERN_CHAPTER =
            "<td>\\s*<a\\s*href=\"(/Manga/[^\"]+)\"\\s*title=\"[^\"]+\">([^<]+)</a>\\s*</td>";
    private static final String PATTERN_SEARCH =
            "href=\"(/Manga/[^\"]+)\">([^<]+)</a>[^<]+<p>[^<]+<span class=\"info\"";
    private static final String PATTERN_MANGA =
            "(https?:[^&|\"]+.ploads/.tc[^&|\"]+).+?href=\"(/.anga/[^\"]+).+?>([^<]+)";

    private static final int[] fltGenre = {
            R.string.flt_tag_4_koma,
            R.string.flt_tag_action,
            R.string.flt_tag_adult,
            R.string.flt_tag_adventure,
            R.string.flt_tag_comedy,
            R.string.flt_tag_comic,
            R.string.flt_tag_cooking,
            R.string.flt_tag_doujinshi,
            R.string.flt_tag_drama,
            R.string.flt_tag_ecchi,
            R.string.flt_tag_fantasy,
            R.string.flt_tag_gender_bender,
            R.string.flt_tag_harem,
            R.string.flt_tag_historical,
            R.string.flt_tag_horror,
            R.string.flt_tag_josei,
            R.string.flt_tag_lolicon,
            R.string.flt_tag_manga,
            R.string.flt_tag_manhua,
            R.string.flt_tag_manhwa,
            R.string.flt_tag_martial_arts,
            R.string.flt_tag_mature,
            R.string.flt_tag_mecha,
            R.string.flt_tag_medical,
            R.string.flt_tag_music,
            R.string.flt_tag_mystery,
            R.string.flt_tag_one_shot,
            R.string.flt_tag_psychological,
            R.string.flt_tag_reincarnation,
            R.string.flt_tag_romance,
            R.string.flt_tag_school_life,
            R.string.flt_tag_sci_fi,
            R.string.flt_tag_seinen,
            R.string.flt_tag_shotacon,
            R.string.flt_tag_shoujo,
            R.string.flt_tag_shoujo_ai,
            R.string.flt_tag_shounen,
            R.string.flt_tag_shounen_ai,
            R.string.flt_tag_slice_of_life,
            R.string.flt_tag_smut,
            R.string.flt_tag_sports,
            R.string.flt_tag_supernatural,
            R.string.flt_tag_time_travel,
            R.string.flt_tag_tragedy,
            R.string.flt_tag_transported,
            R.string.flt_tag_webtoon,
            R.string.flt_tag_yaoi,
            R.string.flt_tag_yuri
    };

    private static final int[] fltStatus = {
            R.string.flt_status_all,
            R.string.flt_status_ongoing,
            R.string.flt_status_completed,
    };
    private static final String[] valStatus = {
            "",
            "Ongoing",
            "Completed",
    };

    private static final int[] fltOrder = {
            R.string.flt_order_rating,
            R.string.flt_order_last_update,
            R.string.flt_order_newest,
            R.string.flt_order_alpha
    };
    private static final String[] valOrder = {
            "/MostPopular",
            "/LatestUpdate",
            "/Newest",
            ""
    };

    KissManga(Context context) {
        super(context);
        setFlag(R.drawable.flag_en);
        setIcon(R.drawable.kissmanga_icon);
        setServerName("KissManga");
        setServerID(KISSMANGA);
    }

    @Override
    public boolean hasList() {
        return true;
    }

    @Override
    public ArrayList<Manga> getMangas() throws Exception {
        return search("");
    }

    @Override
    public ArrayList<Manga> search(String term) throws Exception {
        // make use of AdvanceSearch, more data is then needed
        Navigator nav = getNavigatorAndFlushParameters();

        // do not hide Doujinshi in result
        nav.addHeader("Cookie", "vns_doujinshi=1; ");

        nav.addPost("authorArtist", "");
        nav.addPost("mangaName", term);
        nav.addPost("status", "");
        nav.addPost("genres", "");

        String source = nav.post(HOST + "/AdvanceSearch");

        ArrayList<Manga> searchList;
        Pattern p = Pattern.compile(PATTERN_SEARCH, Pattern.DOTALL);
        Matcher m = p.matcher(source);
        if (m.find()) {
            searchList = new ArrayList<>();
            searchList.add(new Manga(getServerID(), m.group(2), m.group(1), m.group().contains("Status:</span>&nbsp;Completed")));
        } else {
            searchList = getMangasSource(source);
        }
        return searchList;
    }

    @Override
    public void loadChapters(Manga manga, boolean forceReload) throws Exception {
        loadMangaInformation(manga, forceReload);
    }

    @Override
    public void loadMangaInformation(Manga manga, boolean forceReload) throws Exception {
        if (manga.getChapters().isEmpty() || forceReload) {
            String source = getNavigatorAndFlushParameters().get(HOST + manga.getPath());

            // Summary
            manga.setSynopsis(
                    getFirstMatchDefault("<span " + "class=\"info\">Summary:</span>(.+?)</div>", source,
                    context.getString(R.string.nodisponible)));

            // Cover Image
            manga.setImages(
                    getFirstMatchDefault("rel=\"image_src\" href=\"(.+?)" + "\"", source, ""));

            // Author
            manga.setAuthor(getFirstMatchDefault("Author:</span>&nbsp;(.+?)</p>", source, context.getString(R.string.nodisponible)));

            // Genre
            manga.setGenre(getFirstMatchDefault("Genres:</span>&nbsp;(.+?)</p>", source, context.getString(R.string.nodisponible)));

            // Status
            manga.setFinished(source.contains("Status:</span>&nbsp;Completed"));

            // Chapter
            Pattern p = Pattern.compile(PATTERN_CHAPTER, Pattern.DOTALL);
            Matcher matcher = p.matcher(source);
            while (matcher.find()) {
                Chapter chapter = new Chapter(matcher.group(2).replace(" Read Online", ""), matcher.group(1));
                chapter.addChapterFirst(manga);
            }
        }
    }

    @Override
    public String getImageFrom(Chapter chapter, int page) throws Exception {
        assert chapter.getExtra() != null;
        return chapter.getExtra().split("\\|")[page];
    }

    @Override
    public void chapterInit(Chapter chapter) throws Exception {
        if (chapter.getPages() == 0) {
            int pages = 0;
            String source = getNavigatorAndFlushParameters().get(HOST + chapter.getPath().replaceAll("[^!-z]+", ""));
            String ca = getNavigatorAndFlushParameters().get(HOST + "/Scripts/ca.js");
            String lo = getNavigatorAndFlushParameters().get(HOST + "/Scripts/lo.js");
            try (Duktape duktape = Duktape.create()) {
                duktape.evaluate(ca);
                duktape.evaluate(lo);
                Pattern p = Pattern.compile("javascript\">(.+?)<", Pattern.DOTALL);
                Matcher m = p.matcher(source);
                while (m.find()) {
                    if (m.group(1).contains("CryptoJS")) {
                        duktape.evaluate(m.group(1));
                    }
                }

                p = Pattern.compile("lstImages.push\\((.+?\\))\\)", Pattern.DOTALL);
                m = p.matcher(source);
                String images = "";
                String image;
                while (m.find()) {
                    pages++;
                    image = (String) duktape.evaluate(m.group(1) + ".toString()");
                    images = images + "|" + image;
                }
                chapter.setExtra(images);
            }
            chapter.setPages(pages);
        }
    }

    private ArrayList<Manga> getMangasSource(String source) {
        ArrayList<Manga> mangas = new ArrayList<>();
        Pattern p = Pattern.compile(PATTERN_MANGA, Pattern.DOTALL);
        Matcher m = p.matcher(source);
        while (m.find()) {
            Manga manga = new Manga(getServerID(), m.group(3), m.group(2), false);
            manga.setImages(m.group(1));
            mangas.add(manga);
        }
        return mangas;
    }

    @Override
    public ServerFilter[] getServerFilters() {
        return new ServerFilter[]{
                new ServerFilter(
                        context.getString(R.string.flt_include_tags),
                        buildTranslatedStringArray(fltGenre), ServerFilter.FilterType.MULTI),
                new ServerFilter(
                        context.getString(R.string.flt_exclude_tags),
                        buildTranslatedStringArray(fltGenre), ServerFilter.FilterType.MULTI),
                new ServerFilter(
                        context.getString(R.string.flt_status),
                        buildTranslatedStringArray(fltStatus), ServerFilter.FilterType.SINGLE),
                new ServerFilter(
                        context.getString(R.string.flt_order) + " (" + context.getString(R.string.flt_hint_order_unfiltered_only) + ")",
                        buildTranslatedStringArray(fltOrder), ServerFilter.FilterType.SINGLE),
        };
    }

    @Override
    public ArrayList<Manga> getMangasFiltered(int[][] filters, int pageNumber) throws Exception {
        Navigator nav = getNavigatorAndFlushParameters();
        String source;

        // do not hide Doujinshi in result
        nav.addHeader("Cookie", "vns_doujinshi=1; ");

        // no filtering is active - use MangaList (much faster as it is smaller)
        if ((filters[0].length + filters[1].length + filters[2][0]) == 0) {
            String web = HOST + "/MangaList" + valOrder[filters[3][0]] + "?page=" + pageNumber;
            source = nav.get(web);
        }
        // filtering is active, use advanced search (slow, as the whole result set is returned)
        else {
            if (pageNumber > 1) {
                // there is only one result page for the advanced search
                return new ArrayList<>();
            } else {
                nav.addPost("mangaName", "");
                nav.addPost("authorArtist", "");
                for (int i = 0; i < fltGenre.length; i++) {
                    int result = 0;

                    if (Util.getInstance().contains(filters[0], i)) {
                        result++;
                    }
                    if (Util.getInstance().contains(filters[1], i)) {
                        result--;
                    }
                    // include + exclude = 0, include = 1, exclude = -1, no selection = 0
                    // map exclude to "2"
                    if (result < 0) {
                        result = 2;
                    }
                    nav.addPost("genres", Integer.toString(result));
                }
                nav.addPost("status", valStatus[filters[2][0]]);

                source = nav.post(HOST + "/AdvanceSearch");
            }
        }

        return getMangasSource(source);
    }

    @Override
    public boolean needRefererForImages() {
        return false;
    }
}
